import apt_pkg
import collections
import glob
import argparse
import urllib.request
import gzip
import json

def get_seeds():
    with urllib.request.urlopen("http://qa.ubuntuwire.org/ubuntu-seeded-packages/seeded.json.gz") as url:
        with gzip.GzipFile(fileobj=url) as seeded_json:
            return json.load(seeded_json)

def read(root, pocket=""):
    out = {}
    comps = collections.defaultdict(set)
    seeds = collections.defaultdict(set)
    lib = collections.defaultdict(set)
    alls = collections.defaultdict(int)
    srcs ={}
    for component in ["main", "restricted", "universe", "multiverse"]:
        for file in glob.glob(f"{root}/var/lib/apt/lists/*_noble{pocket}_{component}_*Sources"):
            with apt_pkg.TagFile(file) as tf:
                for sec in tf:
                    if sec["Architecture"] == "all":
                        alls[component] += 1
                        continue
                    pkg = sec["Package"]
                    ver = sec["Version"]

                    if any(pkg.startswith(pre) for pre in ("rust-", "haskell-", "golang-")):
                        continue

                    out[pkg] = ver
                    comps[component].add(pkg)

        for file in glob.glob(f"{root}/var/lib/apt/lists/*_noble{pocket}_{component}_*Packages"):
            with apt_pkg.TagFile(file) as tf:
                for sec in tf:
                    src = sec.get("Source", sec["Package"]).split(None, 1)[0] or sec["Package"]
                    bin = sec["Package"]

                    if bin in seeded:
                        seeds[src].update(":".join(seed) for seed in seeded[bin])

                    # Ignore non-task biniares
                    if "Task"  in sec and False:
                        seeds[sec.get("Source", sec["Package"]).split(None, 1)[0] or sec["Package"]].update(set(t.strip() for t in sec["Task"].split(",")))

                    if (sec["Package"].startswith("lib") and not sec["Package"].endswith("-dev")) or "/libs" in sec["Section"] or sec["Section"] == "libs":
                        lib[sec.get("Source", sec["Package"]).split(None, 1)[0] or sec["Package"]].add(sec["Package"])


    for file in glob.glob(f"{root}/var/lib/apt/lists/*_noble{pocket}_InRelease"):
        with apt_pkg.TagFile(apt_pkg.open_maybe_clear_signed_file(file)) as tf:
            for sec in tf:
                if "Date" in sec:
                    date = sec["Date"]


    return out, alls, lib, seeds, comps, date


p = argparse.ArgumentParser()
p.add_argument("--t64", action="store_true")
args = p.parse_args()
seeded = get_seeds()
then, thenall, _, _, _, thendate = read("root2.t64" if args.t64 else "root2")
now, nowall, nowlibs, nowseeds, comps, nowdate = read("root")
proposed, proposedall, _, _, _, proposeddate = read("root", "-proposed")

if args.t64:
    print("# These packages have the same version in the release pocket now as they did when the t64 transition started")
    print("# They may still be using 32-bit times internally, and may even not have frame pointers")
    print("# pre-t64  Release file date:", thendate)
else:
    print("# Rebuilds since -fno-omit-framepointer report (ordered by number of seeds they are in, how many libraries they build, name)")
print("#    read it in curl <url> | less -S to get no wrap around if there's a huge number of seeds in the comments")
print("# pre-fp   Release file date:", thendate)
print("# current  Release file date:", nowdate)
print("# proposed Release file date:", nowdate)
print("# skipped: golang-* haskell-* rust-*")
print()
for component in ["main", "restricted", "universe", "multiverse"]:
    count = sum(1 for package in sorted(comps[component]) if now[package] == then.get(package))
    count_seeds = sum(1 for package in sorted(comps[component]) if now[package] == then.get(package) and nowseeds[package])
    count_libs = sum(1 for package in sorted(comps[component]) if now[package] == then.get(package) and nowlibs[package])
    count_good = sum(1 for package in sorted(comps[component]) if now[package] != then.get(package))
    print(f"# {component}: {count} old packages ({count_seeds} with seeds, {count_libs} with libs), {count_good} good ones, {nowall[component]} arch: all ones")
    for package in sorted(comps[component], key=lambda p: (-len(nowseeds[p]), -len(nowlibs[p]), p)):
        if now[package] == then.get(package):
            if package in proposed:
                print(f"{component}: src:{package:36} # newer version in proposed")
            elif nowseeds[package] and nowlibs[package]:
                print(f"{component}: src:{package:36} # seeds: {' '.join(sorted(nowseeds[package]))}; libs: {' '.join(sorted(nowlibs[package]))}")
            elif nowseeds[package]:
                print(f"{component}: src:{package:36} # seeds: {' '.join(sorted(nowseeds[package]))}")
            elif nowlibs[package]:
                print(f"{component}: src:{package:36} # libs: {' '.join(sorted(nowlibs[package]))}")
            else:
                print(f"{component}: src:{package}")
    print()
